@@ -1,7 +1,6 @@
 /* Evolution calendar - iCalendar component object
  *
- * Copyright (C) 2000 Ximian, Inc.
- * Copyright (C) 2000 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Author: Federico Mena-Quintero <federico@ximian.com>
  *
@@ -22,7 +21,6 @@
 #ifndef E_CAL_COMPONENT_H
 #define E_CAL_COMPONENT_H
 
-#include <glib/gmacros.h>
 #include <time.h>
 #include <glib-object.h>
 #include <libical/ical.h>
@@ -292,7 +290,7 @@ void e_cal_component_get_priority (ECalComponent *comp, int **priority);
 void e_cal_component_set_priority (ECalComponent *comp, int *priority);
 
 void e_cal_component_get_recurid (ECalComponent *comp, ECalComponentRange *recur_id);
-const char *e_cal_component_get_recurid_as_string (ECalComponent *comp);
+char *e_cal_component_get_recurid_as_string (ECalComponent *comp);
 void e_cal_component_set_recurid (ECalComponent *comp, ECalComponentRange *recur_id);
 
 void e_cal_component_get_rdate_list (ECalComponent *comp, GSList **period_list);
@@ -364,7 +362,7 @@ typedef struct _ECalComponentAlarm ECalComponentAlarm;
 /* An alarm occurrence, i.e. a trigger instance */
 typedef struct {
 	/* UID of the alarm that triggered */
-	const char *auid;
+	char *auid;
 
 	/* Trigger time, i.e. "5 minutes before the appointment" */
 	time_t trigger;
