@@ -26,6 +26,12 @@
  */
 
 
+/* This file is broken and suffers from multiple author syndrome. 
+This needs to be rewritten with a lot of functions cleaned up. 
+
+There are a lot of places where code is unneccesarily duplicated, 
+which needs to be better organized via functions */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h> 
 #endif
@@ -89,6 +95,7 @@ static void groupwise_populate_msg_body_from_item (EGwConnection *cnc, CamelMult
 static void groupwise_msg_set_recipient_list (CamelMimeMessage *msg, EGwItem *item);
 static void gw_update_cache ( CamelFolder *folder, GList *item_list, CamelException *ex, gboolean uid_flag);
 static CamelMimeMessage *groupwise_folder_item_to_msg ( CamelFolder *folder, EGwItem *item, CamelException *ex );
+static char* groupwise_get_filename (CamelFolder *folder, const char *uid, CamelException *ex);
 
 
 #define d(x)  
@@ -96,6 +103,16 @@ static CamelMimeMessage *groupwise_folder_item_to_msg ( CamelFolder *folder, EGw
 const char * GET_ITEM_VIEW_WITH_CACHE = "peek default recipient threading attachments subject status priority startDate created delivered size recurrenceKey message notification";
 const char * GET_ITEM_VIEW_WITHOUT_CACHE = "peek default recipient threading hasAttachment subject status priority startDate created delivered size recurrenceKey";
 
+static char* 
+groupwise_get_filename (CamelFolder *folder, const char *uid, CamelException *ex)
+{
+	CamelGroupwiseFolder *gw_folder = CAMEL_GROUPWISE_FOLDER(folder);
+
+	return camel_data_cache_get_filename (gw_folder->cache, "cache", uid, ex);
+}
+
+
+/* Get a message from cache if available otherwise get it from server */
 static CamelMimeMessage *
 groupwise_folder_get_message( CamelFolder *folder, const char *uid, CamelException *ex )
 {
@@ -209,32 +226,46 @@ groupwise_folder_get_message( CamelFolder *folder, const char *uid, CamelExcepti
 	return msg;
 }
 
+/* create a mime message out of an gwitem */
 static void
 groupwise_populate_details_from_item (CamelMimeMessage *msg, EGwItem *item)
 {
+	EGwItemType type;
 	char *dtstring = NULL;
 	char *temp_str = NULL;
 
 	temp_str = (char *)e_gw_item_get_subject(item);
 	if(temp_str)
 		camel_mime_message_set_subject (msg, temp_str);
+	type = e_gw_item_get_item_type (item);
+
+	if (type == E_GW_ITEM_TYPE_APPOINTMENT  || type == E_GW_ITEM_TYPE_NOTE || type == E_GW_ITEM_TYPE_TASK) {
+		int offset = 0;
+		dtstring = e_gw_item_get_start_date (item);
+		time_t actual_time = e_gw_connection_get_date_from_string (dtstring);
+		camel_mime_message_set_date (msg, actual_time, offset);
+		return;
+	}
+
 	dtstring = e_gw_item_get_delivered_date (item);
 	if(dtstring) {
 		int offset = 0;
-		time_t time = e_gw_connection_get_date_from_string (dtstring);
-		time_t actual_time = camel_header_decode_date (ctime(&time), &offset);
+		time_t actual_time = e_gw_connection_get_date_from_string (dtstring);
 		camel_mime_message_set_date (msg, actual_time, offset);
 	} else {
 		time_t time;
 		time_t actual_time;
 		int offset = 0;
 		dtstring = e_gw_item_get_creation_date (item);
-		time = e_gw_connection_get_date_from_string (dtstring);
-		actual_time = camel_header_decode_date (ctime(&time), NULL);
+		if (dtstring) {
+				actual_time = e_gw_connection_get_date_from_string (dtstring);
+		} else
+				actual_time = (time_t) 0;
 		camel_mime_message_set_date (msg, actual_time, offset);
 	}
 }
 
+/* convert an item to a msg body. set content type etc. */
 static void
 groupwise_populate_msg_body_from_item (EGwConnection *cnc, CamelMultipart *multipart, EGwItem *item, char *body)
 {
@@ -299,6 +330,7 @@ groupwise_populate_msg_body_from_item (EGwConnection *cnc, CamelMultipart *multi
 	camel_object_unref (part);
 }
 
+/* Set the recipients list in the message from the item */
 static void
 groupwise_msg_set_recipient_list (CamelMimeMessage *msg, EGwItem *item)
 {
@@ -358,7 +390,7 @@ groupwise_msg_set_recipient_list (CamelMimeMessage *msg, EGwItem *item)
 			}
 		}
 
-		/* README: This entire status-opt code should go away once the new status-tracking code is tested */
+		/* The status tracking code is working fine. someone need to remove this */
 		if (enabled) {
 			camel_medium_add_header ( CAMEL_MEDIUM (msg), "X-gw-status-opt", (const char *)status_opt);
 			g_free (status_opt);
@@ -411,6 +443,7 @@ groupwise_msg_set_recipient_list (CamelMimeMessage *msg, EGwItem *item)
 	}
 }
 
+/* code to rename a folder. all the "meta nonsense" code should simply go away */
 static void
 groupwise_folder_rename (CamelFolder *folder, const char *new)
 {
@@ -519,10 +552,13 @@ free_node (EGwJunkEntry *entry)
 	}
 }
 
+/* This is a point of contention. We behave like the GW client and update our junk list
+in the same way as the GroupWise client. Add senders to junk list */
 static void
 update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 {
 	gchar **email = NULL, *from = NULL;	
+	int index = 0;
 	CamelGroupwiseStore *gw_store= CAMEL_GROUPWISE_STORE(store);
 	CamelGroupwiseStorePrivate  *priv = gw_store->priv;
 	EGwConnection *cnc = cnc_lookup (priv);
@@ -532,18 +568,25 @@ update_junk_list (CamelStore *store, CamelMessageInfo *info, int flag)
 
 	email = g_strsplit_set (from, "<>", -1);
 
-	if (!email || !email[1])
+	if (from [0] == '<') {
+		/* g_strsplit_set will add a dummy empty string as the first string if the first character in the 
+		original string is one of the delimiters that you want to suppress. 
+		Refer to g_strsplit_set documentation */
+		index = 1;
+	}
+
+	if (!email || !email[index])
 		goto error;
 
 	if (flag == ADD_JUNK_ENTRY)
-		e_gw_connection_create_junk_entry (cnc, email[1], "email", "junk");
+		e_gw_connection_create_junk_entry (cnc, email[index], "email", "junk");
 	else if (flag == REMOVE_JUNK_ENTRY) {
 		GList *list = NULL;
 		EGwJunkEntry *entry;
 		if (e_gw_connection_get_junk_entries (cnc, &list)== E_GW_CONNECTION_STATUS_OK){
 			while (list) {
 				entry = list->data;
-				if (!g_ascii_strcasecmp (entry->match, email[1])) { 
+				if (!g_ascii_strcasecmp (entry->match, email[index])) { 
 					e_gw_connection_remove_junk_entry (cnc, entry->id);
 				}
 				list = list->next;
@@ -613,6 +656,7 @@ groupwise_sync_summary (CamelFolder *folder, CamelException *ex)
 	camel_store_summary_save ((CamelStoreSummary *)((CamelGroupwiseStore *)folder->parent_store)->summary);
 }
 
+/* This may need to be reorganized. */
 static void
 groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 {
@@ -664,6 +708,11 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			continue;
 		flags = camel_message_info_flags (info);	
 
+		if (!(flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
+				camel_message_info_free(info);
+				continue;
+		}
+
 		if ((flags & CAMEL_MESSAGE_JUNK) && strcmp(camel_folder_get_name(folder), JUNK_FOLDER)) { 
 			/*marked a message junk*/
 			move_to_junk (folder, info, ex);
@@ -761,6 +810,8 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	
 	CAMEL_GROUPWISE_FOLDER_REC_UNLOCK (folder, cache_lock);
 
+	/* Do things in bulk. Reduces server calls, network latency etc.  */
+
 	if (deleted_items) {
 		CAMEL_SERVICE_REC_LOCK (gw_store, connect_lock);
 		if (!strcmp (folder->full_name, "Trash")) {
@@ -922,7 +973,13 @@ update_update (CamelSession *session, CamelSessionThreadMsg *msg)
 	done = FALSE;
 	m->slist = NULL;
 
-	while (!done && !camel_application_is_exiting) {
+	while (!done) {
+
+		if (camel_application_is_exiting) {
+				CAMEL_SERVICE_REC_UNLOCK (gw_store, connect_lock);
+				return ;
+		}
+
 		item_list = NULL;
 		status = e_gw_connection_get_all_mail_uids (m->cnc, m->container_id, cursor, FALSE, READ_CURSOR_MAX_IDS, position, &item_list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
@@ -1056,6 +1113,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 	char *time_string = NULL, *t_str = NULL;
 	struct _folder_update_msg *msg;
 	gboolean check_all = FALSE;
+	int new_items = 0;
 
 	/* Sync-up the (un)read changes before getting updates,
 	so that the getFolderList will reflect the most recent changes too */
@@ -1150,7 +1208,7 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		   for ( sl = slist ; sl != NULL; sl = sl->next) 
 		   list = g_list_append (list, sl->data);*/
 
-		if (slist && g_slist_length(slist) != 0)
+		if (slist && (new_items = g_slist_length(slist)) != 0)
 			check_all = TRUE;
 
 		g_slist_free (slist);
@@ -1191,22 +1249,37 @@ groupwise_refresh_folder(CamelFolder *folder, CamelException *ex)
 		if (gw_store->current_folder != folder) {
 			gw_store->current_folder = folder;
 		}
-
-		if (list) {
-			gw_update_cache (folder, list, ex, FALSE);
-		}
 		
 		if (check_all && !is_proxy) {
 				EGwContainer *container;
-				container = e_gw_connection_get_container (cnc, container_id);
+				int i=0;
 
-				d(printf ("Evolution's folder summary length is : %u\tserver has %u items",
-										camel_folder_summary_count (folder->summary), e_gw_container_get_total_count (container)));
+				do {
+						/* HACK: Refer to Novell bugzilla bug #464379 */ 
+						container = e_gw_connection_get_container (cnc, container_id);
+						++i;
+						if (!strcmp (folder->full_name, e_gw_container_get_name (container))) 
+								i = 10;
+				} while (i < 2);
 
-				if (camel_folder_summary_count (folder->summary) == e_gw_container_get_total_count (container))
-						check_all = FALSE;
+				if (i == 10) {
+						/* HACK: Refer to Novell bugzilla bug #464379 */ 
+						d(printf ("Evolution's folder summary length is : %u\tserver has %u items",
+												camel_folder_summary_count (folder->summary), e_gw_container_get_total_count (container)));
+
+						if ((camel_folder_summary_count (folder->summary) + new_items) == e_gw_container_get_total_count (container))
+								check_all = FALSE;
+
+						folder->summary->unread_count = e_gw_container_get_unread_count (container);
+						folder->summary->visible_count = e_gw_container_get_total_count (container);
+				} else
+					check_all = FALSE;
 				g_object_unref (container);
 		}
+
+		if (list) {
+			gw_update_cache (folder, list, ex, FALSE);
+		}
 	}
 
 
@@ -1246,6 +1319,7 @@ end1:
 	return;
 }
 
+/* Update the GroupWise cache with the list of items passed. should happen in thread. */
 static void
 gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean uid_flag) 
 {
@@ -1270,7 +1344,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 
 	int folder_needs_caching;
 
-	camel_object_get (folder, NULL, CAMEL_OFFLINE_FOLDER_ARG_SYNC_OFFLINE, &folder_needs_caching, NULL);
+	camel_object_get (folder, NULL, CAMEL_OFFLINE_FOLDER_SYNC_OFFLINE, &folder_needs_caching, NULL);
 
 	changes = camel_folder_change_info_new ();
 	container_id = g_strdup (camel_groupwise_store_container_id_lookup (gw_store, folder->full_name));
@@ -1343,6 +1417,8 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 				mi->info.content->type = camel_content_type_new ("multipart", "mixed");	
 			}
 
+			mi->info.flags = 0;
+
 			if (type == E_GW_ITEM_TYPE_APPOINTMENT || type == E_GW_ITEM_TYPE_TASK || type == E_GW_ITEM_TYPE_NOTE)
 				camel_message_info_set_user_flag ((CamelMessageInfo*)mi, "$has_cal", TRUE);
 		}
@@ -1359,16 +1435,12 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 
 		item_status = e_gw_item_get_item_status (item);
 		if (item_status & E_GW_ITEM_STAT_READ)
-			status_flags |= CAMEL_MESSAGE_SEEN;
+			mi->info.flags |= CAMEL_MESSAGE_SEEN;
 		else 
 			mi->info.flags &= ~CAMEL_MESSAGE_SEEN;
 
 		if (item_status & E_GW_ITEM_STAT_REPLIED)
-			status_flags |= CAMEL_MESSAGE_ANSWERED;
-		if (exists) 
-			mi->info.flags |= status_flags;
-		else 
-			mi->info.flags = status_flags;
+			mi->info.flags |= CAMEL_MESSAGE_ANSWERED;
 
 		priority = e_gw_item_get_priority (item);
 		if (priority && !(g_ascii_strcasecmp (priority,"High"))) {
@@ -1435,26 +1507,27 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 				|| type ==  E_GW_ITEM_TYPE_TASK ) {
 			temp_date = e_gw_item_get_start_date (item);
 			if (temp_date) {
-				time_t time = e_gw_connection_get_date_from_string (temp_date);
-				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
+				time_t actual_time = e_gw_connection_get_date_from_string (temp_date);
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		} else {
 			temp_date = e_gw_item_get_delivered_date(item);
 			if (temp_date) {
-				time_t time = e_gw_connection_get_date_from_string (temp_date);
-				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
+				time_t actual_time = e_gw_connection_get_date_from_string (temp_date);
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			} else {
-				time_t time;
 				time_t actual_time;
 				temp_date = e_gw_item_get_creation_date (item);
-				time = e_gw_connection_get_date_from_string (temp_date);
-				actual_time = camel_header_decode_date (ctime(&time), NULL);
+				if (temp_date) {
+						/* Creation date can be returned as null for auto-generated meetings */
+						actual_time = e_gw_connection_get_date_from_string (temp_date);
+				} else
+					actual_time = (time_t) 0;
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		}
 
+		mi->info.dirty = TRUE;
 		if (exists) {
 			camel_folder_change_info_change_uid (changes, mi->info.uid);
 			camel_message_info_free (pmi);
@@ -1462,12 +1535,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 			mi->info.uid = camel_pstring_strdup (e_gw_item_get_id(item));
 			mi->info.size = e_gw_item_get_mail_size (item);	
 			mi->info.subject = camel_pstring_strdup(e_gw_item_get_subject(item));
-			mi->info.dirty = TRUE;
 			
-			folder->summary->visible_count ++;
-			if (!(mi->info.flags & CAMEL_MESSAGE_SEEN))
-					folder->summary->unread_count ++;
-
 			camel_folder_summary_add (folder->summary,(CamelMessageInfo *)mi);
 			camel_folder_change_info_add_uid (changes, mi->info.uid);
 			camel_folder_change_info_recent_uid (changes, mi->info.uid);
@@ -1512,6 +1580,7 @@ gw_update_cache (CamelFolder *folder, GList *list, CamelException *ex, gboolean
 	camel_folder_change_info_free (changes);
 }
 
+/* Update summary, if there is none existing, create one */
 void
 gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex) 
 {
@@ -1656,22 +1725,21 @@ gw_update_summary ( CamelFolder *folder, GList *list,CamelException *ex)
 		    type ==  E_GW_ITEM_TYPE_TASK ) {
 			temp_date = e_gw_item_get_start_date (item);
 			if (temp_date) {
-				time_t time = e_gw_connection_get_date_from_string (temp_date);
-				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
+				time_t actual_time = e_gw_connection_get_date_from_string (temp_date);
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			} 
 		} else {
 			temp_date = e_gw_item_get_delivered_date(item);
 			if (temp_date) {
-				time_t time = e_gw_connection_get_date_from_string (temp_date);
-				time_t actual_time = camel_header_decode_date (ctime(&time), NULL);
+				time_t actual_time = e_gw_connection_get_date_from_string (temp_date);
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			} else {
-				time_t time;
 				time_t actual_time;
 				temp_date = e_gw_item_get_creation_date (item);
-				time = e_gw_connection_get_date_from_string (temp_date);
-				actual_time = camel_header_decode_date (ctime(&time), NULL);
+				if (temp_date) {
+						actual_time = e_gw_connection_get_date_from_string (temp_date);
+				} else
+						actual_time = (time_t) 0;
 				mi->info.date_sent = mi->info.date_received = actual_time;
 			}
 		}
@@ -1780,7 +1848,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 									gsize len_iter = 0;
 									char *temp = NULL;
 	
-									temp = g_base64_decode(t_attach, &len_iter);
+									temp = (char *) g_base64_decode(t_attach, &len_iter);
 									gstr = g_string_append_len (gstr, temp, len_iter);
 									g_free (temp);
 									len += len_iter;
@@ -1881,6 +1949,8 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 			if (attach->contentid && (is_text_html_embed != TRUE))
 				is_text_html_embed = TRUE;
 
+			/* MIME.822 from server for the weak hearted client programmer */
+
 			if ( (!g_ascii_strcasecmp (attach->name, "TEXT.htm") ||
 			     !g_ascii_strcasecmp (attach->name, "Mime.822") ||
 			     !g_ascii_strcasecmp (attach->name, "Header") ||
@@ -1924,7 +1994,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 								gsize len_iter = 0;
 								char *temp = NULL;
 
-								temp = g_base64_decode(t_attach, &len_iter);
+								temp = (char *) g_base64_decode(t_attach, &len_iter);
 								gstr = g_string_append_len (gstr, temp, len_iter);
 								g_free (temp);
 								len += len_iter;
@@ -1994,6 +2064,8 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 	}/* if attach_list */
 	/********************/
 
+	
+	/* this is broken for translations. someone should care to fix these hacks. nobody except groupwise users are affected though */
 	if (e_gw_item_get_priority (item))
 		camel_medium_add_header ( CAMEL_MEDIUM (msg), "Priority", e_gw_item_get_priority(item));
 
@@ -2100,6 +2172,7 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	
 	item = camel_groupwise_util_item_from_message (cnc, message, CAMEL_ADDRESS (message->from));
 	/*Set the source*/
+	/* FIXME: use flags and avoid such name comparisons in future */
 	if (!strcmp (folder->name, RECEIVED))
 		e_gw_item_set_source (item, "received");
 	if (!strcmp (folder->name, SENT))
@@ -2141,6 +2214,8 @@ groupwise_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	CAMEL_SERVICE_REC_UNLOCK (folder->parent_store, connect_lock);
 }
 
+/* A function to compare uids, inspired by strcmp .
+This code was used in some other provider also , imap4rev1 iirc */
 static int
 uid_compar (const void *va, const void *vb)
 {
@@ -2157,6 +2232,7 @@ uid_compar (const void *va, const void *vb)
 		return 1;
 }
 
+/* move messages */
 static void
 groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids, 
 		CamelFolder *destination, GPtrArray **transferred_uids, 
@@ -2297,6 +2373,7 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			}
 		}
 
+
 		if (destination_is_trash) {
 				e_gw_connection_remove_item (cnc, source_container_id, (const char*) uids->pdata[index]);
 				camel_folder_summary_remove_uid (source->summary, uids->pdata[index]);
@@ -2323,6 +2400,10 @@ groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 								/*if ( !strcmp(source->full_name, SENT) ) {
 								  camel_folder_delete_message(source, uids->pdata[index]);
 								  } else {*/
+
+								if (!(gw_info->info.flags & CAMEL_MESSAGE_SEEN))
+										source->summary->unread_count --;
+
 								camel_folder_summary_remove_uid (source->summary, uids->pdata[index]);
 								camel_folder_change_info_remove_uid (changes, uids->pdata[index]);
 								//}
@@ -2475,6 +2556,7 @@ camel_groupwise_folder_class_init (CamelGroupwiseFolderClass *camel_groupwise_fo
 	camel_folder_class->sync = groupwise_sync;
 	camel_folder_class->expunge = groupwise_expunge;
 	camel_folder_class->transfer_messages_to = groupwise_transfer_messages_to;
+	camel_folder_class->get_filename = groupwise_get_filename;
 }
 
 static void
@@ -2686,7 +2768,21 @@ convert_to_task (EGwItem *item, char **str, int *len)
 	g_string_append_printf (gstr, "UID:%s\n",e_gw_item_get_icalid (item));
 	g_string_append_printf (gstr, "DTSTART:%s\n",e_gw_item_get_start_date (item));
 	g_string_append_printf (gstr, "SUMMARY:%s\n", e_gw_item_get_subject (item));
-	g_string_append_printf (gstr, "DESCRIPTION:%s\n", e_gw_item_get_message (item));
+
+	temp = e_gw_item_get_message (item);
+	if (temp) {
+		g_string_append(gstr, "DESCRIPTION:");
+		while (*temp) {
+			if (*temp == '\n')
+				g_string_append(gstr, "\\n");
+			else
+				g_string_append_c(gstr, *temp);
+			temp++;
+		}
+		g_string_append(gstr, "\n");	
+	}
+	temp = NULL;
+
 	g_string_append_printf (gstr, "DTSTAMP:%s\n", e_gw_item_get_creation_date (item));
 	g_string_append_printf (gstr, "X-GWMESSAGEID:%s\n", e_gw_item_get_id (item));
 	g_string_append_printf (gstr, "X-GWSHOW-AS:BUSY\n");
@@ -2741,6 +2837,7 @@ convert_to_note (EGwItem *item, char **str, int *len)
 	EGwItemOrganizer *org = NULL;
 	GString *gstr = g_string_new (NULL);
 	char **tmp = NULL;
+	const char *temp = NULL;
 
 	tmp = g_strsplit (e_gw_item_get_id (item), "@", -1);
 
@@ -2750,7 +2847,21 @@ convert_to_note (EGwItem *item, char **str, int *len)
 	g_string_append_printf (gstr, "UID:%s\n",e_gw_item_get_icalid (item));
 	g_string_append_printf (gstr, "DTSTART:%s\n",e_gw_item_get_start_date (item));
 	g_string_append_printf (gstr, "SUMMARY:%s\n", e_gw_item_get_subject (item));
-	g_string_append_printf (gstr, "DESCRIPTION:%s\n", e_gw_item_get_message (item));
+
+	temp = e_gw_item_get_message (item);
+	if (temp) {
+		g_string_append(gstr, "DESCRIPTION:");
+		while (*temp) {
+			if (*temp == '\n')
+				g_string_append(gstr, "\\n");
+			else
+				g_string_append_c(gstr, *temp);
+			temp++;
+		}
+		g_string_append(gstr, "\n");	
+	}
+	temp = NULL;
+
 	g_string_append_printf (gstr, "DTSTAMP:%s\n", e_gw_item_get_creation_date (item));
 	g_string_append_printf (gstr, "X-GWMESSAGEID:%s\n", e_gw_item_get_id (item));
 	g_string_append_printf (gstr, "X-GWRECORDID:%s\n", tmp[0]);
