@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@novell.com>
  *
- *  Copyright 2004 Novell, Inc. (www.novell.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -52,9 +52,9 @@ enum {
 };
 
 struct _CamelGroupwiseJournalEntry {
-	EDListNode node;
+	CamelDListNode node;
 	
-	int type;
+	guint32 type;
 
 	char *uid;
 	char *original_uid;
