@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-source.h
  *
- * Copyright (C) 2003  Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Ettore Perazzoli <ettore@ximian.com>
  */
@@ -66,8 +66,6 @@ ESource *e_source_new_from_standalone_xml (const char *xml);
 
 ESource *e_source_copy (ESource *source);
 
-gboolean e_source_equal (ESource *source_1, ESource *source_2);
-
 gboolean  e_source_update_from_xml_node  (ESource    *source,
 					  xmlNodePtr  node,
 					  gboolean   *changed_return);
@@ -119,8 +117,12 @@ void         e_source_foreach_property (ESource *source,
 					GHFunc func,
 					gpointer data);
 
+char *e_source_get_duped_property (ESource *source, const char *property);
 char *e_source_build_absolute_uri (ESource *source);
 
+gboolean e_source_equal (ESource *a, ESource *b); 
+gboolean e_source_xmlstr_equal (const gchar *a, const gchar *b);
+
 G_END_DECLS
 
 #endif /* _E_SOURCE_H_ */
