@@ -4,14 +4,25 @@
 #ifndef __E2K_VALIDATE_H_
 #define __E2K_VALIDATE_H_
 
+#include <gtk/gtk.h>
+
 #ifdef __cplusplus
 extern "C" {
 #pragma }
 #endif /* __cplusplus */
 
+typedef enum {
+	E2K_AUTOCONFIG_USE_GAL_DEFAULT, /* try ntlm if available and then basic if not or failed */
+
+	E2K_AUTOCONFIG_USE_GAL_BASIC,
+	/*E2K_AUTOCONFIG_USE_GAL_SASL,*/
+	E2K_AUTOCONFIG_USE_GAL_NTLM
+} E2kAutoconfigGalAuthPref;
+
 typedef struct {
         char *host;
         char *ad_server;
+	E2kAutoconfigGalAuthPref ad_auth;
         char *mailbox;
         char *owa_path;
 	gboolean is_ntlm;
@@ -36,7 +47,11 @@ typedef enum {
 	E2K_AUTOCONFIG_FAILED
 } E2kAutoconfigResult;
 
-gboolean e2k_validate_user (const char *owa_url, char *key, char **user, ExchangeParams *exchange_params, gboolean *remember_password, E2kAutoconfigResult *result);
+gboolean e2k_validate_user (const char *owa_url, char *key, char **user,
+			    ExchangeParams *exchange_params,
+			    gboolean *remember_password,
+			    E2kAutoconfigResult *result,
+			    GtkWindow *parent);
 
 
 #ifdef __cplusplus
